﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3
#pragma version=1.40
#pragma igorVersion=7
#pragma IndependentModule=UserProcLoader
#include <Resize Controls>

strconstant ksPackageName=UserProcLoader
strconstant ksPrefsFileName=UserProcLoaderPrefs.bin
constant kPrefsVersion=100

constant kResizablePanel=1 

// If you're using Igor Pro 8 or later, this package can check 
// periodically for new releases if you've installed Project Updater. 
// http: www.igorexchange.com/project/Updater 
static constant kCheckForUpdates=1  // set to 1 to check for updates on init
static strconstant ksShortTitle="UserProcLoader" // the project short title on IgorExchange
static constant kProjectID=21127 // the project node on IgorExchange

// https://www.wavemetrics.com/user/tony

// Provides an easy way to load and unload procedure files from the User 
// Procedures or Wavemetrics Procedures folders.

// Requires Igor Pro 8 for full functionality.
// In Igor 7 it's possible to select procedures that cannot be unloaded

// Trying to load a procedure that won't compile could lead to confusion.
// Version 1.20 allows following of shortcuts/aliases, and is tolerant of 
// semicolons in filenames. Repopulating the procedure lists, however, is
// significantly slower.

// Settings
// Fast file search option is intolerant of semicolons in filenames.
// Mouseover listbox items to see path to file and compiler directives 
// (#pragma & #include lines) displayed as listbox help. 

menu "Macros"
	"Load or Unload...",/Q, makeProcLoaderPanel()
end

function makeProcLoaderPanel()
	
	if (CheckForUpdates())
		return 0 // this file has been updated; quit execution to allow user to reload
	endif
	
	DoWindow /k LoaderPanel
	NewDataFolder /O root:Packages
	NewDataFolder /O root:Packages:procLoader
	DFREF dfr=root:Packages:procLoader
	
	// load package preferences
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	
	// prepare waves for two listboxes			
	make /o /n=0 dfr:w_loadedSelection /WAVE=w_loadedSelection, dfr:w_unloadedSelection /WAVE=w_unloadedSelection
	make /o /n=0/T dfr:w_loadedProcs, 	dfr:w_unloadedProcs, dfr:w_loadedTitle, dfr:w_unloadedTitle, dfr:w_loadedProcsHelp, dfr:w_unloadedProcsHelp
	wave /SDFR=dfr/T w_loadedProcs, w_unloadedProcs, w_loadedTitle, w_unloadedTitle, w_loadedProcsHelp, w_unloadedProcsHelp
	w_loadedTitle={"loaded"}; w_unloadedTitle={"unloaded"}
				
	// create panel	
	NewPanel /K=1/N=LoaderPanel/W=(prefs.win.left,prefs.win.top,prefs.win.left+500,prefs.win.top+360) as "Load or Unload Procedures"
	//ModifyPanel /W=LoaderPanel fixedSize=1, noedit=1
	ModifyPanel /W=LoaderPanel noedit=1
	
	// popup to select procedures folder on disk
	PopupMenu popupFolder, win=LoaderPanel, pos={20, 10}, value="User Procedures;Wavemetrics Procedures"
	PopupMenu popupFolder, win=LoaderPanel, proc=loaderPopMenuProc 
	
	// Button to open Procedure Browser window (useful if you want to open procedure after loading)
	Button buttonBrowser, win=LoaderPanel, pos={430, 15}, size={15,14}, picture=pb, labelBack=0, title=""
	Button buttonBrowser, win=LoaderPanel, proc=loaderButtonProc, help={"Open Procedure Browser"}
	
	// Button for settings
	Button buttonSettings, win=LoaderPanel, pos={460, 14}, size={15,15}, picture=cog, labelBack=0, title=""
	Button buttonSettings, win=LoaderPanel, proc=loaderButtonProc, help={"Change Settings"}
	
	// listbox for procedure files in folder that are not currently loaded
	ListBox listboxUnloaded,win=LoaderPanel, pos={20, 40},size={200, 270}, listwave=w_unloadedProcs, selwave=w_unloadedSelection, mode=9
	ListBox listboxUnloaded,win=LoaderPanel, fsize=12, proc=loaderListBoxProc, titleWave=w_unloadedTitle, helpWave=w_unloadedProcsHelp
	
	// buttons to be enabled when a valid selection is made from a listbox
	Button buttonLoad win=LoaderPanel, pos={230,120},size={40,40},title=">", proc=loaderButtonProc, disable=2
	Button buttonUnload win=LoaderPanel, pos={230,170},size={40,40},title="<", proc=loaderButtonProc, disable=2
	
	// listbox for procedure files in folder that ARE currently loaded
	ListBox listboxLoaded,win=LoaderPanel, mode=9, selwave=w_loadedSelection, pos={280, 40},size={200, 270}, editstyle=1
	ListBox listboxLoaded,win=LoaderPanel, listwave=w_loadedProcs,fsize=12, proc=loaderListBoxProc, titleWave=w_loadedTitle, helpWave=w_loadedProcsHelp
	
	// popup menus to choose sort order for listboxes
	PopupMenu popupUnloadedSort, win=LoaderPanel, pos={20, 320}, value="Sort By Name;Sort by Path;Sort By Creation Date;Sort By Modification Date"
	PopupMenu popupUnloadedSort, win=LoaderPanel, proc=loaderPopMenuProc, mode=prefs.sortUnloaded
	PopupMenu popupLoadedSort, win=LoaderPanel, pos={280, 320}, value="Sort By Name;Sort by Path;Sort By Creation Date;Sort By Modification Date"
	PopupMenu popupLoadedSort, win=LoaderPanel, proc=loaderPopMenuProc, mode=prefs.sortLoaded
	
	if(kResizablePanel)
		// resizing userdata for controls
		PopupMenu popupFolder,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,BY!!#;-!!#@^!!#<pz!!#](Aon\"Qzzzzzzzzzzzzzz!!#](Aon\"Qzz"
		PopupMenu popupFolder,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#u:Duafnzzzzzzzzzzz"
		PopupMenu popupFolder,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#u:Du]k<zzzzzzzzzzzzzz!!!"
		Button buttonBrowser,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,I=!!#<(!!#<(!!#;mz!!#o2B4uAezzzzzzzzzzzzzz!!#o2B4uAezz",title="",size={15.00,14.00},pos={430.00,15.00}
		Button buttonBrowser,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
		Button buttonBrowser,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#u:Du]k<zzzzzzzzzzzzzz!!!"
		Button buttonSettings,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,IL!!#;m!!#<(!!#<(z!!#o2B4uAezzzzzzzzzzzzzz!!#o2B4uAezz",title="",size={15.00,15.00},pos={460.00,14.00}
		Button buttonSettings,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
		Button buttonSettings,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#u:Du]k<zzzzzzzzzzzzzz!!!"
		ListBox listboxUnloaded,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,BY!!#>.!!#AW!!#BAz!!#](Aon\"Qzzzzzzzzzzzzzz!!\";f=(u2eBE/#4z"
		ListBox listboxUnloaded,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
		ListBox listboxUnloaded,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
		Button buttonLoad,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,H!!!#@T!!#>.!!#>.z!!#`-A7TLfzzzzzzzzzzzzzz!!#`-A7TLfzz"
		Button buttonLoad,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#AtDKKH1zzzzzzzzzzz"
		Button buttonLoad,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#AtDKKH1zzzzzzzzzzzzzz!!!"
		Button buttonUnload,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,H!!!#A9!!#>.!!#>.z!!#`-A7TLfzzzzzzzzzzzzzz!!#`-A7TLfzz"
		Button buttonUnload,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#AtDKKH1zzzzzzzzzzz"
		Button buttonUnload,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#AtDKKH1zzzzzzzzzzzzzz!!!"
		ListBox listboxLoaded,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,HG!!#>.!!#AW!!#BAz!!\";f=(u2eBE/#4zzzzzzzzzzzzz!!#o2B4uAeBE/#4z"
		ListBox listboxLoaded,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
		ListBox listboxLoaded,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
		PopupMenu popupUnloadedSort,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,BY!!#BZ!!#@<!!#<pz!!#](Aon\"Qzzzzzzzzzzzzzz!!#](Aon\"Qzz"
		PopupMenu popupUnloadedSort,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
		PopupMenu popupUnloadedSort,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
		PopupMenu popupLoadedSort,win=LoaderPanel,userdata(ResizeControlsInfo)= A"!!,HG!!#BZ!!#@<!!#<pz!!#o2B4uAezzzzzzzzzzzzzz!!#o2B4uAezz"
		PopupMenu popupLoadedSort,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
		PopupMenu popupLoadedSort,win=LoaderPanel,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
		
		// resizing userdata for panel
		SetWindow LoaderPanel,userdata(ResizeControlsInfo)= A"!!*'\"z!!#C_!!#Bnzzzzzzzzzzzzzzzzzzzzz"
		SetWindow LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzzzzzzzzzzzzzzz"
		SetWindow LoaderPanel,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzzzzzzzzz!!!"
		
		// resizing panel hook
		SetWindow LoaderPanel hook(ResizeControls)=ResizeControls#ResizeControlsHook
	endif
	
	// populate listbox waves
	updateListboxWaves()
	return 1
end

function loaderListBoxProc(lba) : ListBoxControl
	STRUCT WMListboxAction &lba
	
	DFREF dfr=root:Packages:procLoader	
	
	if(datafolderRefStatus(dfr)==0)
		return 0
	endif
	
	if(lba.eventCode==-1) // clean up
		KillWindow /Z LoaderPrefsPanel	
		SaveWindowPosition(lba.win)			
		KillDataFolder /Z dfr
		return 0
	endif
	
	string procWinStr=""
	
	if (lba.eventCode==4 && lba.eventMod&2^4) // right click in listbox cell
		if (stringmatch(lba.ctrlName, "listboxUnloaded"))
			return 0 // allow context menu for loaded files
		endif
		procWinStr=lba.listWave[lba.row]+".ipf"
		wave /T w_normalProcs=ListToTextWave(winlist("*",";","WIN:128,INDEPENDENTMODULE:0"),";")
		string popupStr
		if (isInTextWave(procWinStr,w_normalProcs)==0)
			// cell contains an independent module, user will have to find another way to open :)
			popupStr="(show "+ procWinStr
		else
			popupStr="show "+ procWinStr
		endif
		PopupContextualMenu popupStr
		if(V_flag)
			DisplayProcedure/W=$procWinStr
		endif
		return 0
	endif	
	
	if(lba.eventCode==3 ) // double click
		if (stringmatch(lba.ctrlName, "listboxUnloaded") && lba.row>-1)
			IncludeListboxSelection(1) // load procs in unloaded listbox on double click
		else // try to display the proc window on double click in loaded listbox
			procWinStr=lba.listWave[lba.row]+".ipf"
			DisplayProcedure/W=$procWinStr
			GetWindow /Z $procWinStr active
			if (V_Value==0) // didn't succeed in displaying proc
				doIgorMenu "Windows", "Procedure Browser"
			endif
		endif
	endif
	
	wave /SDFR=dfr w_loadedSelection, w_unloadedSelection
	
	// are we selecting procedures to load or to unload?
	variable load=stringmatch(nameofwave(lba.selWave), "w_unloadedSelection")
	
	if(lba.eventCode==4 || lba.eventCode==5)
		if(load && numpnts(w_loadedSelection))
			w_loadedSelection=w_loadedSelection & ~1 
			w_loadedSelection=w_loadedSelection & ~8 // clear selection in other listbox
		endif
		if(load==0 && numpnts(w_unloadedSelection))
			w_unloadedSelection=0
		endif
		// don't allow disabled cells to be selected
		// (can be done by shift-click or using arrow keys)
		lba.selWave = (lba.selWave&129)==129 ? lba.selWave & ~1 : lba.selWave
		lba.selWave = (lba.selWave&136)==136 ? lba.selWave & ~8 : lba.selWave
	endif
	
	// enable buttons based on selection	
	updateButtons()	
	return 0
end

function loaderPopMenuProc(pa)
	STRUCT WMPopupAction &pa

	if(pa.eventCode==2)
		updateListboxWaves()
	endif	
	return 0
end

function loaderButtonProc(ba)
	STRUCT WMButtonAction &ba
	
	if(ba.eventCode!=2)
		return 0
	endif
	
	if(stringmatch(ba.ctrlName, "buttonBrowser"))
		doIgorMenu "Windows", "Procedure Browser"
	elseif(stringmatch(ba.ctrlName, "buttonSettings"))	
		STRUCT PackagePrefs prefs
		LoadPrefs(prefs)
		
		string checked= "\\M0:!" + num2char(18) + ":" // checkmark code
		string items=selectstring((prefs.options&1),"", checked)
		items+="Use Fast Search;"
		items+=selectstring((prefs.options&2),"", checked)
		items+="Show File Path as Help;"
		items+=selectstring((prefs.options&4),"", checked)
		items+="Scan for Compiler Directives;"
		PopupContextualMenu items
		variable itemSelected=V_flag
		switch( itemSelected )
			case 1:
				prefs.options+= (prefs.options&1) ? -1 : 1
				break;
			case 2:
				prefs.options+= (prefs.options&2) ? -2 : 2
				break;
			case 3:
				prefs.options+= (prefs.options&2) ? -4 : 4
				break;
		endswitch		
		SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
		if(itemSelected>1)
			updateListboxWaves()
		endif
	elseif(stringmatch(ba.ctrlName, "buttonLoad"))
		IncludeListboxSelection(1)
	elseif(stringmatch(ba.ctrlName, "buttonUnload"))
		IncludeListboxSelection(0)
	endif
	return 0	
end

function updateButtons()
	
	DFREF dfr=root:Packages:procLoader	
	wave /SDFR=dfr w_loadedSelection, w_unloadedSelection
		
	variable selections=0
	if (numpnts(w_unloadedSelection))
		selections=sum(w_unloadedSelection)
	endif
	Button buttonLoad win=LoaderPanel, disable=2*(selections==0)
	// examine first bit of each point in w_loadedSelection
	selections=0
	if (numpnts(w_loadedSelection))
		Duplicate /free w_loadedSelection w_testSel
		w_testSel=w_testSel&1	
		selections=sum(w_testSel)
	endif
	Button buttonUnload win=LoaderPanel, disable=2*(selections==0)
end

function IncludeListboxSelection(insert)
	variable insert // 1 for InsertInclude, 0 for DeleteInclude
	
	DFREF dfr=root:Packages:procLoader
	if(insert)
		wave w_sel=dfr:w_unloadedSelection
		wave /T w_list=dfr:w_unloadedProcs
	else
		wave w_sel=dfr:w_loadedSelection
		wave /T w_list=dfr:w_loadedProcs
	endif
	string cmd=selectString(insert, "DELETEINCLUDE", "INSERTINCLUDE")
	string cmd2=""	
	ControlInfo /W=LoaderPanel popupFolder
	variable isWMproc=v_value-1
	string startquote=selectString(isWMproc, "\"", "<")
	string endquote=selectString(isWMproc, "\"", ">")
	string strExct="", procList=""
	variable i
	
	for(i=0;i<numpnts(w_sel);i+=1)
		if( (w_sel[i]&2^0) || (w_sel[i]&2^3) )
			sprintf strExct, "%s %s%s%s", cmd, startquote, w_list[i], endquote 			
			Execute/P/Q/Z strExct
			procList=addlistItem(w_list[i],procList)
		endif
	endfor
	sprintf cmd2, "#selectItems(%g,\"%s\")", insert, procList
	Execute/P/Q/Z "COMPILEPROCEDURES "	
	Execute/P/Q/Z getindependentModuleName()+"#updateListboxWaves()" // queue an update for the listboxes	
	Execute/P/Q/Z getindependentModuleName()+cmd2 // select the items that have just been loaded/unloaded
	return 1	
end

// search recursively for ipf files and return list of files as textwave
// slow method avoids IndexedFile(path, -1, string) and IndexedDir(path, -1, string) 
// because semicolon is valid character filenames!
function /wave getProcsRecursive(fast)
	variable fast
		
	variable maxFolders=1000, fullpath=1
	variable isWin=stringmatch(StringByKey("OS", igorinfo(3))[0,2],"Win")
	variable folderIndex, i
	string strFile="", strFolder=""		
	make /free/T/n=0 w_folders, w_allFiles
	
	w_folders={SelectedFolder()} // search recursively starting in this folder

	// w_folders will grow as subfolders are added
	for(folderIndex=0;folderIndex<numpnts(w_folders);folderIndex+=1)
		// check files in folderIndexth folder from w_folders		
		newPath /O/Q/Z tempPathFF, w_folders[folderIndex]		
		
		if(fast) // don't worry about semicolons in filenames, or shortcuts ending in .ipf!
			// add list of ipf files in current folder to w_allFiles
			wave /T w=ListToTextWave(IndexedFile(tempPathFF, -1, ".ipf"),";")
			if (fullpath)
				w=w_folders[folderIndex]+w
			endif
			TextWaveConcatenate(w_allFiles, w)
			// add list of folders in current folder to w_folders
			wave /T w=ListToTextWave(IndexedDir(tempPathFF, -1, 1),";")
			w+=":"
			TextWaveConcatenate(w_folders, w)
			
			if(isWin)
				// get a list of shortcuts 
				wave /T w=ListToTextWave(IndexedFile(tempPathFF, -1, ".lnk"),";")
				if(numpnts(w))
					w=ResolveAlias(w_folders[folderIndex]+w)
					Duplicate /free w w_f 
					// keep shortcuts to files in w, shortcuts to folders in w_f
					if(fullpath==0)
						w = ParseFilePath(0, w, ":", 1, 0)
					endif
					TextWaveStringMatch(w, "*.ipf")
					TextWaveConcatenate(w_allFiles, w)
					TextWaveStringMatch(w_f, "*:")
					TextWaveConcatenate(w_folders, w_f)
				endif
			else // mac			
				// get a list of file aliases
				wave /T w=ListToTextWave(IndexedFile(tempPathFF, -1, "alis"),";")
				if(numpnts(w))
					w=ResolveAlias(w_folders[folderIndex]+w)
					if(fullpath==0)
						w = ParseFilePath(0, w, ":", 1, 0)
					endif
					TextWaveStringMatch(w, "*.ipf")
					TextWaveConcatenate(w_allFiles, w)
				endif
				// get a list of folder aliases
				wave /T w=ListToTextWave(IndexedFile(tempPathFF, -1, "fdrp"),";")
				if(numpnts(w))
					w=ResolveAlias(w_folders[folderIndex]+w)
					TextWaveZapBlanks(w)
					TextWaveConcatenate(w_folders, w)
				endif			
			endif
			continue // next folder
		endif
				
		// the slow way: loop through files in folder
		for(i=0;1;i+=1)
			strFile=IndexedFile(tempPathFF, i, "????")		
			if(strlen(strFile)==0)
				break
			endif
			GetFileFolderInfo /Q/Z w_folders[folderIndex]+strFile
			if(V_isAliasShortcut)
				strFile=ResolveAlias(w_folders[folderIndex]+strFile)
				if(stringmatch(strFile,"*:"))
					w_folders[numpnts(w_folders)]={strFile}
				endif
				if(stringmatch(strFile,"*.ipf"))
					if(fullpath==0)
						strFile=ParseFilePath(0, strFile, ":", 1, 0)
					endif
					w_allFiles[numpnts(w_allFiles)]={strFile}
				endif
				continue // next file
			endif
			
			if(stringmatch(strFile,"*.ipf"))
				if(fullpath)
					strFile=w_folders[folderIndex]+strFile
				endif
				w_allFiles[numpnts(w_allFiles)]={strFile}
				continue
			endif
		endfor // next file
		
		// add subfolders in current folder
		for(i=0;1;i+=1)
			strFolder=IndexedDir(tempPathFF, i, 1)
			if(strlen(strFolder)==0)
				break
			endif
			w_folders[numpnts(w_folders)]={strFolder+":"}
		endfor // next subfolder
		if(folderIndex==maxFolders)
			print "Warning: getProcsRecursive exceeding max iterations"
			break
		endif
	endfor // next folder
	
	KillPath /Z tempPathFF
	return w_allFiles
end

// appends w2 to w
function TextWaveConcatenate(w, w2)
	wave /T w, w2
	
	variable oldPnts=numpnts(w), newPnts=numpnts(w2)	
	if(newPnts)
		Redimension /n=(oldPnts+newPnts), w
		w[oldPnts,]=w2[p-oldPnts]
	endif
end

// keep only points of textwave matching strMatch
function TextWaveStringMatch(w, strMatch)
	wave /T w
	string strMatch
	
	if(numpnts(w))
		w=selectstring(stringmatch(w,strMatch),"",w)
		TextWaveZapBlanks(w)
	endif
end

// Recursively resolve shortcuts to files/directories
// returns full path or an empty string if the file does not exist or the
// shortcut points to a non existing file/folder
function/S ResolveAlias(strPath)
	string strPath // full path to file, folder or alias

	GetFileFolderInfo/Q/Z removeEnding(strPath, ":")
	if(V_isAliasShortcut)
		return ResolveAlias(S_aliasPath)
	endif
	if(v_flag==0) 
		return strPath // path to a file or folder
	endif
	return ""
end

// returns path to folder selected in popup
// 1: User Procedures 2: Wavemetrics Procedures
function /S SelectedFolder()
	
	ControlInfo /W=LoaderPanel  popupFolder
	if (V_flag<1)
		return ""
	endif
	if(v_value==1)
		return SpecialDirPath("Igor Pro User Files",0,0,0)+"User Procedures:"
	endif
	return SpecialDirPath("Igor Application",0,0,0)+"Wavemetrics Procedures:"
end

// strip trailing stuff in the output from winlist
function /S filenameFromString(s)
	string s
	
	if (numtype(strlen(s))==2)
		return ""
	endif
	
	SplitString/E=".*\.ipf" s // remove module names
	return removeending(s_value, ".ipf")
end

// extract quoted filename from include statement
function /S filenameFromIncludeStatement(s)
	string s

	if (numtype(strlen(s))==2)
		return ""
	endif
	
	ControlInfo /W=LoaderPanel popupFolder
	variable isWMproc=v_value-1
	string startquote=selectString(isWMproc, "\"", "<")
	string endquote=selectString(isWMproc, "\"", ">")
	
	variable start, stop
	start=strsearch(s, startquote, 0)
	stop=strsearch(s, endquote, start+1)
	if (start==-1 || stop==-1)
		return ""
	endif
	return s[start+1,stop-1]
end

// truth that string s is a member of textwave w
function isInTextWave(s, w)
	string s
	wave /T w
	
	if(numtype(strlen(s))==2)
		return 0
	endif
	
	FindValue /TEXT=s/TXOP=4 /Z w		
	return (V_Value>-1)
end

// Remove empty points from textwave w (and optionally the corresponding 
// points of waves in wave reference wave w_zapRefs). Does not check 
// numbers of points in w_zapRefs waves.
function TextWaveZapBlanks(w [, w_zapRefs])
	wave /T w
	wave /WAVE w_zapRefs
	
	if(numpnts(w)==0)
		return 0
	endif
	variable i=0, j=0
	do
		if(stringmatch(w[i],""))
			DeletePoints i,1,w
			if(waveexists(w_zapRefs))
				for(j=0;j<numpnts(w_zapRefs);j+=1)
					DeletePoints i,1,w_zapRefs[j]
				endfor
			endif
		else	
			i+=1
		endif
	while(i<numpnts(w))
	return numpnts(w)
end

function /S getProcFilePath(strWin)
	string strWin
	
	GetWindow /Z $strWin file
	return stringFromList(1, s_value)+stringFromList(0, s_value)	
end

function updateListboxWaves()
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
		
	DFREF dfr=root:Packages:procLoader
	wave /SDFR=dfr w_loadedSelection, w_unloadedSelection
	variable numLoaded, numUnloaded, numIncluded
	variable UnloadedSortOrder, LoadedSortOrder
	variable testing=0
	string strIgnore=""
	string strIgnoreList="~$*;" // list of filename patterns to ignore
	// IndexedFile ignores names strating with "._" but may find OS-hidden files
	variable i, imax
	
	ControlInfo /W=LoaderPanel popupUnloadedSort	
	UnloadedSortOrder=v_value-1 // 0 by name, 1 by path, 2 by creation, 3 by mod
	ControlInfo /W=LoaderPanel popupLoadedSort	
	LoadedSortOrder=v_value-1
	
	// create a free text wave containing a list of procedure files (full filepath)
	// in the selected source folder	
	wave /T w_unloadedPaths=getProcsRecursive(prefs.options&1)
	numUnloaded=numpnts(w_unloadedPaths)
	make /free/n=(numUnloaded)/T  w_unloadedProcs
	
	// reduce file path to filename without extension
	w_unloadedProcs=parseFilePath(3, w_unloadedPaths, ":", 0, 0)
	
	// remove names that match patterns in the ignore list
	imax=itemsinlist(strIgnoreList)
	for(i=0;i<imax;i+=1)
		strIgnore=StringfromList(i, strIgnoreList)
		w_unloadedProcs=selectstring(stringmatch(w_unloadedProcs, strIgnore), w_unloadedProcs, "")
	endfor
	
	// create a free text wave containing a list of all procedure files in the experiment
	// use colon as list separator because semicolon is legal in filenames
	wave /T w_loadedProcs=ListToTextWave(winlist("*",":","WIN:128,INDEPENDENTMODULE:1"),":")	
	numLoaded=numpnts(w_loadedProcs)
	
	make /free/n=(numLoaded)/T w_loadedPaths
	w_loadedPaths=getProcFilePath(w_loadedProcs)

	// clean up names
	if(numLoaded)		
		w_loadedProcs=FilenameFromString(w_loadedProcs) // some points may be blank
	endif		
	
	// remove procedures that aren't in selected source folder from loaded list
	make /free/N=1/WAVE w_toZap
	if(numLoaded)		
		w_loadedProcs=selectstring(isInTextWave(w_loadedProcs[p], w_unloadedProcs),"",w_loadedProcs[p])
		w_toZap=w_loadedPaths
		numLoaded=TextWaveZapBlanks(w_loadedProcs, w_zapRefs=w_toZap)
	endif
	
	// remove already loaded procedures from unloaded list
	if(numUnloaded)
		w_unloadedProcs=selectstring(isInTextWave(w_unloadedProcs[p], w_loadedProcs),w_unloadedProcs[p],"")
		w_toZap=w_unloadedPaths
	  	numUnloaded=TextWaveZapBlanks(w_unloadedProcs, w_zapRefs=w_toZap)
	endif	
	
	make /free/n=(numLoaded)/T w_loadedProcsHelp
	make /free/n=(numLoaded)/D w_loadedProcsDate
	make /free/n=(numUnloaded)/T w_unloadedProcsHelp
	make /free/n=(numUnloaded)/D w_unloadedDate	
	
	w_unloadedProcsHelp=getHeader(w_unloadedPaths, prefs.options)
	w_loadedProcsHelp=getHeader(w_loadedPaths, prefs.options)
	
	if(unloadedSortOrder>1)
		w_unloadedDate=getFileDate(w_unloadedPaths, unloadedSortOrder-1)
	endif
	if(loadedSortOrder>1)
		w_loadedProcsDate=getFileDate(w_loadedPaths, loadedSortOrder-1)
	endif
	
	// figure out which procedures can be unloaded.
	// using INCLUDE:2 with winlist gets all included files, 
	// not just those that we can DELETEINCLUDE
	
	// load procedure file lines into text wave
	wave /T w_includedProcs=ListToTextWave(ProcedureText("",0,"procedure"), "\r")
	
	// find lines that start with #include
	grep /E="^#include[\s]*[<\"]"/Z w_includedProcs as w_includedProcs
	numIncluded=TextWaveZapBlanks(w_includedProcs)
	
	// extract file names from between quotes
	if(numIncluded)
		w_includedProcs=filenameFromIncludeStatement(w_includedProcs)
	endif
  	
  	if(LoadedSortOrder>1 && numpnts(w_loadedProcsDate)==numLoaded) // sort by creation/mod date
  		sort/R w_loadedProcsDate, w_loadedProcsDate, w_loadedProcs, w_loadedProcsHelp, w_loadedPaths
  	elseif(LoadedSortOrder==1) // sort by path
  		sort w_loadedPaths, w_loadedProcs, w_loadedProcsHelp, w_loadedPaths // not essential to sort paths here
	else // sort by name
		sort w_loadedProcs, w_loadedProcs, w_loadedProcsHelp, w_loadedPaths
	endif	
  	
  	if(UnloadedSortOrder>1 && numpnts(w_unloadedDate)==numUnloaded)
  		sort/R w_unloadedDate, w_unloadedDate, w_unloadedProcs, w_unloadedProcsHelp, w_unloadedPaths
  	elseif(UnloadedSortOrder==1)
		sort w_unloadedPaths, w_unloadedProcs, w_unloadedProcsHelp, w_unloadedPaths
  	else
		sort w_unloadedProcs, w_unloadedProcs, w_unloadedProcsHelp, w_unloadedPaths
	endif
	
	// copy free waves into the package folder
	Duplicate /O/T w_unloadedProcs dfr:w_unloadedProcs
	Duplicate /O/T w_unloadedProcsHelp dfr:w_unloadedProcsHelp
		
	Duplicate /O/T w_loadedProcs dfr:w_loadedProcs
	Duplicate /O/T w_loadedProcsHelp dfr:w_loadedProcsHelp
		
	if(testing)
		Duplicate /O w_unloadedDate dfr:w_unloadedDate
		Duplicate /O w_unloadedPaths dfr:w_unloadedPaths
		Duplicate /O w_loadedProcsDate dfr:w_loadedProcsDate
		Duplicate /O w_loadedPaths dfr:w_loadedPaths
	endif
	
	Redimension /N=(numLoaded) w_loadedSelection
	Redimension /N=(numUnloaded) w_unloadedSelection
	
	// disable cells containing procedures that cannot be unloaded using DELETEINCLUDE	
	if(numLoaded)
		w_loadedSelection=(1-isInTextWave(w_loadedProcs, w_includedProcs))*2^7
	endif
	if(numUnloaded)
		w_unloadedSelection=0
	endif	
	
	// nothing is selected, so disable load and unload buttons
	Button /Z buttonLoad win=LoaderPanel, disable=2
	Button /Z buttonUnload win=LoaderPanel, disable=2
	return 1
end

// options bit 1: full path to file, bit 2: #pragma and #include lines
// ignore bit 0
function /s getHeader(str_file, options)
	string str_file
	variable options
	
	if(options<2)
		return ""
	endif
	
	if(strlen(str_file)==0)
		return "no file"
	endif
	
	variable maxLines=20 // number of lines in each file to search for compiler directives
	variable refNum, i
	string strHeader="", strLine=""
	
	if(options&2)
		string UserFiles=SpecialDirPath("Igor Pro User Files",0,0,0)
		string IgorApp=SpecialDirPath("Igor Application",0,0,0)
		if(stringmatch(str_file, UserFiles+"*"))
			strHeader=str_file[strlen(UserFiles), inf]+"\r"
		elseif(stringmatch(str_file, IgorApp+"*"))
			strHeader=str_file[strlen(IgorApp), inf]+"\r"
		else
			strHeader=str_file+"\r"
		endif
	endif
	
	if(options&4)
		open /R/Z refnum as str_file
		if(refnum==0)
			return strHeader+"cannot open"
		endif
		for (i=0;i<maxLines;i+=1)
			FReadLine refNum, strLine
			if(strlen(strLine)==0)
				break
			elseif(stringmatch(strLine, "#*"))
				strHeader+=strLine		
			endif
		endfor	
		close refnum
	endif
	strHeader=selectString((strlen(strHeader)>0), "no pragma", strHeader)
	return removeending(strHeader, "\r")	
end

function getFileDate(strFile, modOrCreation)
	string strFile; variable modOrCreation // 0: creation, 1: mod
		
	GetFileFolderInfo /Q/Z strFile
	return modOrCreation ? V_modificationDate : V_creationDate	
end

// called after updateListboxWaves() to select the listbox items in selList
function selectItems(whichBox, selList)
	variable whichBox // O: unloaded, 1: loaded
	string selList
	
	string strListBox=selectString(whichBox, "listboxUnloaded", "listboxLoaded")
	
	DFREF dfr=root:Packages:procLoader	
	if(whichBox==1)
		wave selWave=dfr:w_loadedSelection
		wave /T listWave=dfr:w_loadedProcs
	else
		wave selWave=dfr:w_unloadedSelection
		wave /T listWave=dfr:w_unloadedProcs	
	endif
	
	variable i, numItems
	numItems=itemsinlist(selList)
	for (i=0;i<numItems;i+=1)
		FindValue /TEXT=stringFromList(i, selList) /Z listWave
		if(V_value>-1)
			selWave[v_value]=1
		endif
	endfor
	ListBox $strListBox,win=LoaderPanel,row=-1
	updateButtons()
end

// ------------------ package preferences --------------------

static structure PackagePrefs
	uint32 version
	uint16 options
	uchar sortUnloaded
	uchar sortLoaded
	STRUCT Rect win // window position and size, 8 bytes		
	char reserved[128-4-2-1-1-8]	
endStructure

// set prefs structure to default values
function SetDefaultPrefs(prefs)
	STRUCT PackagePrefs &prefs
	
	prefs.version=kPrefsVersion
	prefs.win.left=20
	prefs.win.top=20
	prefs.win.right=20+500
	prefs.win.bottom=20+360
	prefs.options=7
	prefs.sortUnloaded=1
	prefs.sortLoaded=1
	
	variable i
	for(i=0;i<(128-16);i+=1)
		prefs.reserved[i]=0
	endfor
end

function LoadPrefs(prefs)
	STRUCT PackagePrefs &prefs
	
	LoadPackagePreferences ksPackageName, ksPrefsFileName, 0, prefs	
	if (V_flag!=0 || V_bytesRead==0 || prefs.version!=kPrefsVersion)
		SetDefaultPrefs(prefs)
	endif
end

// save window position in package prefs
function SaveWindowPosition(strWin)
	string strWin
	
	STRUCT PackagePrefs prefs
	LoadPrefs(prefs)
	
	string recreation=winRecreation(strWin, 0)
	variable index, top, left, bottom, right
	// save window position
	index=strsearch(recreation, "/W=(", 0)
	if (index>0)
		sscanf recreation[index,strlen(recreation)-1], "/W=(%g,%g,%g,%g)", left, top, right, bottom
		prefs.win.top=top
		prefs.win.left=left 
		prefs.win.bottom=bottom
		prefs.win.right=right
	endif
	
	ControlInfo /W=LoaderPanel popupUnloadedSort	
	prefs.sortUnloaded=v_value
	ControlInfo /W=LoaderPanel popupLoadedSort	
	prefs.sortLoaded=v_value
	
	SavePackagePreferences ksPackageName, ksPrefsFileName, 0, prefs
end
 
// ------------------ Button proc pictures -----------------------

// PNG: width= 90, height= 28
Picture pb
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"&!!!!=#R18/!9'oUErZ1?&TgHDFAm*iFE_/6AH5;7DfQssEc39jTBQ
	=U#OG#M5u_NK`L;'g'_`?_5JPn)_d:C()V]%S0h9%3pu4-+e;tk,")Zj,2$t(u:^^!26PP,C#]8!<9
	mWaQ8X-SSBWG/hC(;G*#S&af\J(Vo(h3i=JdEa20h9%Bm_&NRbPIltcZl$<dZ3HKn$CpGHi)smgJt<
	2g*[<'Xa;<t!BdsaPsOO"h=u=#@$P%=i-3?c*Ydi'%Ab)j+3>HX.,7PocZV:W6E^l,aImEkaM]:*'m
	bL`L6pfSJJc63O+3ANeuTl'kA,hOWe93S-G[q*-P6FrcH'TnrT4.>cLlR_KAXsdKjN",$]&0nO+B&R
	">RL]CN;`9._&d][6h'7(^>`1g1:YEMd]WVWriKZkEi[R2%1^O9gfP#?=&qC%a9eZ(_EhMq:"*r[P(
	KYl.*a,BQsH@RWK&3?uap)g+'eI@0A8Pps!/h+sK/DUP6@0D;B_m<dJ-;i7BXaD/V`L"ShK%.6s9po
	k[6<D&EoQbc$+'I\kk\>,>;ai(")=K[B&aRSE%6V,0L-n&sp<gjpr?RaQf%,in^6N4(uiYXT#H/=d<
	="ari;0Dbn$'\G6r;P6kA]QME!=TX0"2;/TMK\=g09eO;AaD^g4$Rg.Y$Or)TjQC*,J.Z>J1TU05@M
	f319f>^S[jh"m35c&+[WA_9$^@M3L;Cr>M0ud?+mCG\D)[i!6,-,m$C_6lSZh0D=ZLU`^-jE(L_/a+
	@f3_e&j>Z+=DRP`\rK#QD&4D5Whg)JX6sW6K#Te`(hHA'h=\a7nTbMA)&$++2'5n%3),AA>ckQTTDn
	[.T'-Fq5Ptp$CkW#c*j.arR/OEDrcr-lYIG0j,k7=iA&VI'^(DJ2.>=jc*pe%TXfX21WitUGK?%te4
	%&aEV)R%LR&uYtK$hc9'lJ(K_DRcVlV"d84.NhbMo_HN78f(sE/-nMMo!p_PPO:X(_.>Mk+uG"-8.R
	WPVW-g@/EC]CSaC>1;k6[M;iS52`f"sQc\`]A>7m@%\'1=+PaWqkMMU>3+#\%bW-',UhH[38oH,Q="
	_UMT\pp3]\I#$;cf3KSY5,^QcpuT(,I#S_&),;?$A@f7XJEo4d:R20_fV17S<CsP'_WLbc,36(1<TC
	k;?Gpi(D7*l>)m1(cR/r&,M(gf0e.1\fs>\Nr;6cL'X^#Nq23XINefNoU6ZRT;&.r>INp.<*Bdb3LR
	g=leH?k^JTssf?)=[jEsr;H]%PBL@fE8TbM)"'uYHIpj>ZC*B%+!p\T"nD"rBLe6_`D=nNZ3^)<7#%
	;#1$Kt@qJT.\7Kn$cWe0fM1',%mr013<Fogs5p1]k;5\:80CqI:\0FYp0=H7k5//)cJLPgpjK,ggo"
	jY1;'bQ8o#EDPObMCT^UF&Ja]lCeZcfkm_3GW+P^2\!rjjEP'+B6tgBGTPFn2<9UoLi"+*5h!Qkb>P
	s<*&L@,dRGb.&RD1V0_s+rJTeuh=B!thc/qRN!\f=HMIX_?uoS,-tn\t)"ZY"XcJX:ZkNH!U-N;/4k
	L:]G4-HpC!W2FJsCN.uBbmek'%s-+MBKcj&k@TCW\ZYfWHsN.5f/V]i`8B"%h.C[:"&KWihV1-(:#U
	#)`o5:L8oMIW*WCSN0fG<^$g2NTai!4^FLeiZ\V5qX/hh$nO1NeAqUNmr"Hno\W2m$h+$1CQDX2=[X
	$8jTD(Tb41&[\doMohnS)K*Gp)kP1Ir$1p2Z0:>:k`A30o?;/LU5sih&JJ'N.Uu?0%ZGS*ol%D10,u
	6bi)KSLqs""Va2GDEX'Qf[eEV$@YhsbOr;Goh%a1n^;RP*;-g?-3`jRUs&6h:l+('.88kD/L\/*K<@
	HiI?G(#$s*3QKE\^+jFk\g_mf"%tr'Z;(TgKJXRs1.'D>SMI->,)ZdWe3"W-"*ToAiq?lB+KniR<3f
	n0KAW?=$BP`As/"\EpZNTu>krVp8.gZCFe0PBm,7ou\bt\fp2G0E'`I^jJBuEkU*WfVrXG[/auCY9M
	heaJ?pV\9N9LKc:C[GiUhe)<lC?,H/A&LJt+O\5[m2Of<&r=S;h#k%mCQQo8r#8G!&^_JZoq9qa)rK
	2(h)J3/Ws_a[I(?cJ5'7laq)RH,VHmVitd)k-R"l8@kmXt_3<bI5[aRp:mDp1`+:ZF<+E4c,u3A[6;
	c+[]1_Z$Q),m+kUYi<+G;Fo)YGOV*OH:LhN+kBZZiI;M`<hY.o]G!QUZ7hP2ak3+0_n9AE11OCH].P
	%J^?U\I''4EL0M2>Q9k2oV=C"n]T<p<VFY'IY"=JZ]\?4R^tOt$>#)[m`ARae]<8r@fMNeg#VN:#3:
	+.li7!9QK=<%k8*_@me^MDp9b#PR*o&l6/!M6iJFE&b`X,-b>ga,15K^H#Dl='`Ri]r0YI/RPb\ieO
	Bd-9[9M4kgk=MrX!dSlOV(fk.LoZ/%(uAE,'!8n:Ts$&Qc'SN]VM9E`=]MgE@M9!@+MP)f7SP>=igc
	r<olZR@9f)mh7V0mBWc7)bQ[Y3]ASqdKZ(=6o5_QS2\h0farNU)f1d*h4>TeCWS#.b\A:3I5Xr$j3E
	:f7@8?k%T]s^!E))hE_N)5lR9V<)jpY)]LYJ6&'s>q`j;pX7,UPN#_lu!!#SZ:.26O@"J
	ASCII85End
End

// PNG: width= 90, height= 30
Picture cog
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"&!!!!?#R18/!3BT8GQ7^D&TgHDFAm*iFE_/6AH5;7DfQssEc39jTBQ
	=U%adj95u_NKeXCo&'5*oW5;QerS2ddE(FP;'6Ql#oBiuS1dW/'TBu9uWoJd,<?@,Ku[OT_*C6>N&V
	5fgKO-Z!0m]BHZFE$kSY/b+).^UIV80KE-KEW"FQ9_,eF^bT7+UPEE3;1R;IlL+VJZGMtc/)57aYSB
	X^5kLMq7Sp9q0YBG@U214ph]9b)&eA4@(!#Ip%g0/F0_UH>\E'7M.=ui]mI6`m'G"T)B'OA[IVmQDm
	=?Eat`-Qd[5bR1c#F]`cT(Se*XgJen[DI=<Y8YF7N!d-X+iY2pq;SVDt;siWS_bs$!E]WT^pRhs]aH
	M%csq\akmJ4tMo<HGU1WqT,5cib%XrHM`>*]/&U:](RQ7QDj\u&#cMNi8>`?8;-?rCcXX>+1^gW10F
	n!h-2G-M;R\a+TD2op'?`R]"%W!DpJmO],i(@-(/F?,;L7Vkt%YOa,f\8+CV@lHVOEMgMZPnh$6>!V
	MTYBm^8f[O,?Z$2MnH6.T'JWSM4HtSissRo-):4d:ecoe5Tn^(gUEQm+o94@;L(/[91%aXk:!pP;mm
	\kh$s.7qbe%=-p1eBtDs*CHp:+$CUY\0A,jM0:4J2&pY-HWBG?nb`"BE/M-#*)+E?I*C/(r;J]APNh
	3!Ea<(u)`o?0R`ma=QV<n?GV/s3:I0Wf2_M0p@:__T%OEl+sL@10K8&ViQgR(0Q3qMLYA':/iba:,;
	]Y$@ACMV&9b[fD4A`Vq5+A!37VD0na`;0#fWNWKq#f5N>Mt)$S['[2:?=(p2$Q$$NX_cXoJ`iVOcHm
	Rb+"_b#*b4@tp)Xq9r*1_<^IVlpMJ=kE>MhiHa2]]q9<d*4(lA_8$4ej2NM5Z!#`oc=+Ttk-]%D5"O
	Yiu,o$V/I<=@2fN3Ds,PNfIEnqn6C?^[OYDs4q2k*s6TFu+@1>SKUmdko@B5>Pp)-]8`l_Ig,/1c.T
	K'Z+asa)qDc*mqZAKmijlOd;;&H$MEMWY1:\q<G#aaNVlho?TWCGL35!G658MH$RpQ,/[:S#==eP-@
	T+s%'h-7&.0)\eW6j@1gNW'FYlgRIid1g1dP0.MtL)"o@*4A+B&XU\9bRSJWg?B!keI%b6T7FS'?W(
	@7j-a-n[,Adkh%U((6"oN9G"iRV$rmb0"2lqXk08!N&V_b13Oo*tc)h=[L]E@W<ihr:]%Dbs-*cCbc
	T^`<b81D1(d_gue7JX)rMl-Q!ag!0.a4mbCL5'MQH2X<p3`1nA#69QNiWDnN[J^:kIm`JPCXo#W8lo
	?KFN_dOMp#7s\i!;q:1Vb`q^Za5j'0Sg8ALVn\tm:OM*.G/IF;=K4S+O/0U]^`u\O,i'69Y*0'f,SH
	:Kp)mH3.EQ3hDf%?f;W[LbJbR7R5Lb$8U4I7,[8ZM7fU7H5(>6BlSlr1cG6"263q!"YH.\`>aLYN(!
	e3hZYm666$Mq_c(_GHO?%CE(rnI-UV=I6M\e$%CXt$`9q"IB8d`/4e)0&Dcf`43oobf6MqdV?d>\73
	X/dI(2jZ+#[Nt'sQg5KRFl_^rEVX>cZ5h3g1gR#*IhTZf+KqmsVrW[`UcE6>MM*N0fMcT[S!:h'=ju
	EaL@5>OfU]WXd+d/JISLYCQZ_BPkB$IiAYUBq1l^ecC4a8EYJ'WJ,pak5V59k6$F23m\(d</D&W$.c
	&:n64N(\`j0%h;m7iB=j)(R^kh8BUCfn<.JP[26K1F7\>0JOc56jWCOX(6k=i$m^+A%<G5QZ.i$e8]
	01cU/k$R?&@c[1P\L>tK[OkSMmQ7lT_puI&4&#-'R99q+p;3\Sn`Ic2G%kDj/"N6oB<E0?Z6Kl"@,Y
	?4P5G,Nu\q^LTs(Vj(g3YCoZEl;W9!T)>ePG,S!10n_Y\rP(FBr90.J3;jVP^MYp<-ML6>(=)*bFsB
	&uWX-VWg'<P8G@$+\A?J1@G'F,/Z.<q/;.,=8I?;gFp>>;IjEQPE_:7`.R+3bElA@DB6<k@ksJ9lg(
	=CVM=g<G(^E#SiiFHZ8.qF-^ppkE&\[U*_);<'Lfk*Fq]^#W339=Q'IWpj"KgUWU]1,ETW<&G^OtOH
	MkGSX1r0@A!=Gd&>m2;$s(nF8b!K?8R`eZjO8V38_I$<1AcnP!)B*`T#3.X=hE[s8PMoFf*/H2@32u
	N'Len`GUk?n:K*@=9gMMi1ES9gF7flQcCD`2n^,h:`S5=GNQ^G#@^/a:?]W`PV50n4Xue>QVk8E1=]
	lWKB?uV(SiXjL_hVC,G-+W-bHd)[C`^u(BPM:VV58ltJcZ8d$CEhp-Ek)Qb.'^'f)4eT`-]7*:O[1>
	WO?>HRRZ3%&Aum4gNS.jU=.^S*B#H\'0H3Z)tG>O;)WW1_IO09[>@PI4Y3R7J0]^!UgQhj+u1-,O_#
	4q7aj2?Y5.V]ph=D*J`g8?n%JH:8P)K%MLrfVTs(X1]A:d+mFtdNBG"";'8siHNZC4&bKJq`%mNb7r
	SW;=`2-+n=L)HDOsFHoS$CX_6m<3W758iRSt7"9?7u`s%5^"&NsfV]&fKiR),nod"F>!-ZZj257RH'
	BCpkU2>pgE:BYW?=m,Fa:(R[)N0g+8U_d1?h6?87.J?1.S8QM+N.?c/5H^[K9Qq/KSe*09PFi*)kOs
	Cpa8151hB![K\`b9:/2t#`0h)TQGGW^_mOCaj@jCA@uU*q95,uIW@7!X&<O\"P/_=YGg"!\PP(fcMs
	XX8]B01I/5GVm/oHal.qLkA^e?r`*cq5<6cMYpEN]i/%/Vl(p'^fIMd8C\oHLnT1!)7oQm]mKeID*1
	Nn<=:2#ZkEe#YpHHH,[1j)T%7I4;@/)4cu_Q1)PbZM:[@i"UEI%;r>p03XoVZ_RkU=+$'7#=USG/bL
	8-+m<=>h,iqN=A8kNQ3E0-ce+TlUO7L$\:&5CW07\^Y5(=Lpj3bn5fXf]+hD?H]7Wq?&[-c"7hNK0#
	/)B'Mj<HT8ma3CurC,*bh\'aHR7Z[QbX-ZmAk800m):lpO:?P+(!;TY'R\j"AmjWGZ^'4n^bf?W3J@
	>&NBK1HqWkVhSJ@2:%U$9,hq:d,G1cCmI-TcrP\J*Ut[2@6?BcK3XN6]^DH?sm>]m;PWk0+t]M3*pb
	_i5ToaNr1&dko4ib1O7G-^#a3R58IWd+6c;6ULrU<E06&]A7B"H::^+p=jM"Cht@E-\k9W-F%RN7[f
	g9`s&hll-^kfa*7KZfMd!Yc("^(?tb@(G_h8BF>IEA!<RhTlND,!db&r!Y3m>2$M&6dHhJmnR;"(TN
	7k9deNFM:rt_%M:_\bI5MO2h<A0N#R:ZSrC"&ps\iY*%&:=-;@IrX+"G9!l_&sOI?=_'7)$hsk)[Og
	CfZ9V$@mNB]AS#G_>V6^Z_/)$iF?19\*_+U8'Lh!@O$@74\ogtP<K2K,l#,B;0e6NJ*#oS35C1Gqa?
	[/#EMYbdp\'g083t^Hm.OC']=?TCh[+\@'/CD^$lb9k>s0TnKIaqh4fI!&lDq*\4*U*,*??/2AnId;
	.P@%q^Y_gV7L#<Y@CP!Nm,EKn=&i8<r@!PTa5]H_PQ](fBqn;iMUE,Pl5E-<#!*W9G0Gifi6\]*<o4
	FnfrX,WF^_\F$1nF]^fU-bLO!=bm%5lG.k3$IWMqUu'c@l,R*B4I#7$5RGsBAo;S"q!W&r%8C2/"PK
	bsa<4<J73edT0;DW#W4)GM@uFDONL"9R+_N^([T!(k&'aJc*VLHV&^R;!(_#4Z$g7@#[rnEd4b][m6
	"s2Cf6FX#Yth+![-Bc9;DCc35!#ZOe]>R5l%A3s9r*"E3cZ^HAq!@!X3ZLR*.A7oQ8om.\p[kV)RYJ
	YS_VJ,ke(!91A):`eg`H3<A0s%C//=2RBq,pC2:S]*\P@U_Oa*3.T[g!Hf.uI$^Z574:Ig+a&Rh&b)
	g:i!IBPVCY]Y$?Mc-nM/==coe'#A=jP*M;$C2,4.LP+[KAA[:ZiG]VW6ipmf;5gRtUogbYmG#,M=Y+
	23\;kLm>:;.qMlKqn)uClJ![.i',6Vo?VRu"<5C39QHia.rIUXNcq/492/E<t4:q=5j]#0#BT^2C8R
	r=7NOaA&EGCimE'I"(o(b72sE7cRPmWBnj]tHh/;(=(Hs(iTH$*GN/REC6P4"-OQ)1Z*S9OlNXqYG,
	/qFo"%Xt5WcH)DMDo_@'gn2mp\l)\(b!Y0Pa!2n,Nj%-R@Yjn?WT$E#t(FUbj=.R08ON,:0qYL%:/M
	0]<Q1)2)G':0@s*h8ZZ<4MLPu4'A3d&SI6l9j+cCI%0ltDh?G(&13<0N\Q=?t??;p9Q8$59_n3Rli=
	5b`N"1`$#>;[JNr+$!*^fli%qGlHBaGd#r_gndbH0<a<pR0sE5L0:j)I_t)\EH/7Wqt8QJMd<r<&WK
	8J3cuoH9hij#22_bS-?/1q+bUC@(DjDc_1Dg*LCYK([C$_m"OB=44C54XF6CiRHM)#JSik-Qi#lgdX
	C:AAV;n0(-%L_0m!T,"d5MVG@HhTKZ87K.p%WH^Xh3kCpXcTY<@p]%p!H!PcGm8Ma`dWI-i:%O`.@A
	\EMhGlp>Rk7O<G2m`*r,h[u\8;4r,bUaQp%EDN*J]D4B1hFXuppq^tpModARV5%<QlNN?L%hAEkIlW
	/#`^]Bs#-d.f-97RH2#l<o@ZXZ&T?iRH-<%N9SU+'so7AAgW2mil0fWaM)A,6@)4Rp@WFC0@Y,uIN:
	5uCJkMPAJFd6VVd/UR6[rI=?0fVgq/kI,s^(YARDN54WD\PD#"bV<C5XKS<`]Fql#m@)ul]O#Nn]-2
	[lB);<HM,sPARkJs:;d4_S!/nh?qGe7@I:AsnMi7DjM_D$2XW>fsY^ZQI8$;`nm!f"mg^^mq'BNs/!
	!!!j78?7R6=>B
	ASCII85End
End

// ----------------  code used to check for updates  ---------------------

// code will work with version 3 of the updater project

static function CheckForUpdates()
	
	if (kCheckForUpdates==0)
		return 0
	endif
	
	variable /G V_updated=0
	string cmd=""
	sprintf cmd, "V_updated=Updater#UpdateCheck(\"%s\")", FunctionPath("") // Updater is independent module
	Execute /Q/Z cmd // will fail quietly for Updater versions prior to 3 in ProcGlobal namespace
	variable isUpdated=V_updated
	KillVariables /Z V_updated
	return isUpdated 
end
